package de.stefandraegersoftware.gui.mvc;

import de.stefandraegersoftware.SerialCommunication;
import jssc.SerialPortException;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.FARBE_WAEHLEN_TITEL;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.FRAGE;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.WOLLEN_SIE_DIE_ANWENDUNG_WIRKLICH_BEENDEN;
import static de.stefandraegersoftware.gui.mvc.Model.BLUE;
import static de.stefandraegersoftware.gui.mvc.Model.GREEN;
import static de.stefandraegersoftware.gui.mvc.Model.RED;
import static javax.swing.JOptionPane.QUESTION_MESSAGE;
import static javax.swing.JOptionPane.YES_NO_OPTION;

/**
 * Klasse Controller.
 * Controller Klasse des MVC Patterns.
 * Dient zum Steuern der registrierten Views über das Observer Pattern.
 */
public class Controller {

    /**
     * Die View des MVC Pattern.
     */
    private final LEDFrame frame;

    /**
     * Das Model des MVC Pattern.
     */
    private Model model;

    /**
     * Das {@link java.util.logging.Logger} Objekt.
     */
    private static Logger logger = Logger.getLogger("Controller");

    /**
     * Konstruktor.
     */
    public Controller() {
        this.frame = new LEDFrame();
        this.model = new Model();
        this.model.addObserver(this.frame);

        addListener();
        initDefaultValues();
    }

    /**
     * Befüllt die {@link javax.swing.JComboBox} mit den Standardwerten aus dem Model.
     */
    private void initDefaultValues() {
        for (String s : this.model.getAvailablePortnames()) {
            this.frame.getAvailableComPortsComboBox().addItem(s);
        }

        if (this.model.getAvailablePortnames().isEmpty()) {
            this.frame.showNoSerialPortWasFoundMessage();
        }

        for (Integer i : this.model.getAvailableBaudrate()) {
            this.frame.getAvailableBaudrateComboBox().addItem(i);
        }

        for (Integer i : this.model.getAvailableDatabits()) {
            this.frame.getAvailableDatabitsComboBox().addItem(i);
        }

        for (Integer i : this.model.getAvailableStopbits()) {
            this.frame.getAvailableStopbitsComboBox().addItem(i);
        }

        for (Integer i : this.model.getAvailableParity()) {
            this.frame.getAvailableParityComboBox().addItem(i);
        }
    }

    /**
     * Setzt die {@link java.awt.event.ActionListener} und {@link java.awt.event.ItemListener} für die Komponenten der View.
     */
    private void addListener() {
        this.frame.setColorChooserListener(new ColorChooserActionListener());
        this.frame.setSendListener(new SendBtnActionListener());
        this.frame.setCloseBtnListener(new CloseBtnActionListener());
        this.frame.setPortComboBoxKeyListener(new ConnectionParameterComboBoxItemListener(ConnectionParameter.PORTNAME));
        this.frame.setBaudrateComboBoxListener(new ConnectionParameterComboBoxItemListener(ConnectionParameter.BAUDRATE));
        this.frame.setDatabitsComboBoxListener(new ConnectionParameterComboBoxItemListener(ConnectionParameter.DATABITS));
        this.frame.setStopbitsComboBoxListener(new ConnectionParameterComboBoxItemListener(ConnectionParameter.STOPBITS));
        this.frame.setParityComboBoxListener(new ConnectionParameterComboBoxItemListener(ConnectionParameter.PARITY));
        this.frame.setTimerTextFieldListener(new TimerNumberFieldKeyListener());
        this.frame.setTestBtnListener(new TestBtnActionListener());
    }

    /**
     * Zeigt die View.
     */
    public final void showView() {
        this.frame.setVisible(true);
    }

    /**
     * Klasse ColorChooserActionListener implementiert das Interface {@link java.awt.event.ActionListener}.
     * Diese Klasse stellt die funktionalität für die Auswahl der Farbe bereit.
     */
    class ColorChooserActionListener implements ActionListener {

        /**
         * @param e das {@link java.awt.event.ActionEvent}
         * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
         */
        public void actionPerformed(final ActionEvent e) {
            JColorChooser colorChooser = new JColorChooser(Color.white);
            Color color = colorChooser.showDialog(null, FARBE_WAEHLEN_TITEL, null);
            Map<String, Integer> colorMap = new HashMap<String, Integer>();
            colorMap.put(RED, color.getRed());
            colorMap.put(GREEN, color.getGreen());
            colorMap.put(BLUE, color.getBlue());
            model.setColorMap(colorMap);
        }
    }

    /**
     * Klasse SendBtnActionListener implementiert das Interface {@link java.awt.event.ActionListener}.
     * Diese Klasse stellt die funktionalität für das Senden des Strings an den Arduino bereit.
     * Der Arduino hat ein Programm welches ein String entgegennimmt und verarbeitet.
     */
    class SendBtnActionListener implements ActionListener {

        /**
         * @param e das {@link java.awt.event.ActionEvent}
         * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
         */
        public void actionPerformed(final ActionEvent e) {
            try {
                if (!isColorMapValid() || model.getSleep() == 0) {
                    JOptionPane.showConfirmDialog(null, "Es muss eine Farbe gewaehlt werden und die Zeit muss > 0 millisekunden sein!", "Fehler!", JOptionPane.OK_CANCEL_OPTION);
                    frame.highlightTimerTextField(model.getSleep() == 0);
                    return;
                }

                SerialCommunication serialCommunication = new SerialCommunication.SerialCommunicationBuilder(model.getPortname()).setSerialPortParameter(model.getBaudrate(), model.getDatabits(), model.getStopbits(), model.getParity()).setSleepForClosePortEvent(model.getSleep()).build();
                Map<String, Integer> colorMap = model.getColorMap();
                int redValue = colorMap.get(RED);
                int greenValue = colorMap.get(GREEN);
                int blueValue = colorMap.get(BLUE);

                DecimalFormat decimalFormat = new DecimalFormat("000");
                String value = String.format("sr%sg%sb%st%se", decimalFormat.format(redValue), decimalFormat.format(greenValue), decimalFormat.format(blueValue), model.getSleep());
                String msg = String.format("send [%s] to %s \'Baudrate: %d, Databits: %d, Stopbits: %d, Parity: %d\' sleepModus:%d", value, model.getPortname(), model.getBaudrate(), model.getDatabits(), model.getStopbits(), model.getParity(), model.getSleep());
                logger.log(Level.INFO, msg);
                serialCommunication.openPort();
                serialCommunication.writeString(value);
                serialCommunication.closePort();
            } catch (SerialPortException e1) {
                e1.printStackTrace();
            }
        }

        /**
         * Validiert ob die {@link java.util.Map} valide Werte für die erstellung einer Farbe beinhaltet.
         *
         * @return liefert {@link Boolean#TRUE} wenn die {@link java.util.Map} valide Werte für die erstellung einer Farbe beinhaltet, andernfalls {@link Boolean#FALSE}
         */
        private boolean isColorMapValid() {
            Map<String, Integer> colorMap = model.getColorMap();
            return (colorMap != null) && colorMap.containsKey(RED) && colorMap.containsKey(GREEN) && colorMap.containsKey(BLUE);
        }
    }

    /**
     * Klasse CloseBtnActionListener implementiert das Interface {@link java.awt.event.ActionListener}.
     * Diese Klasse stellt die funktionalität für die Auswahl der Farbe bereit.
     */
    class CloseBtnActionListener implements ActionListener {

        /**
         * @param e das {@link java.awt.event.ActionEvent}
         * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
         */
        public void actionPerformed(final ActionEvent e) {
            int result = JOptionPane.showConfirmDialog(frame, WOLLEN_SIE_DIE_ANWENDUNG_WIRKLICH_BEENDEN, FRAGE, YES_NO_OPTION, QUESTION_MESSAGE);
            if (result == JOptionPane.YES_OPTION) {
                System.exit(0);
            }
        }

    }

    /**
     * Klasse TestBtnActionListener implementiert das Interface {@link java.awt.event.ActionListener}.
     * Diese Klasse stellt die funktionalität für die Auswahl der Farbe bereit.
     */
    class TestBtnActionListener implements ActionListener {

        /**
         * @param e das {@link java.awt.event.ActionEvent}
         * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
         */
        public void actionPerformed(final ActionEvent e) {
            try {
                SerialCommunication serialCommunication = new SerialCommunication.SerialCommunicationBuilder(model.getPortname()).setSerialPortParameter(model.getBaudrate(), model.getDatabits(), model.getStopbits(), model.getParity()).build();
                serialCommunication.openPort();
                serialCommunication.closePort();
            } catch (SerialPortException ex) {
                logger.log(Level.WARNING, ex.getMessage());
            }
        }

    }

    /**
     * ENUM mit den Verbindungsparameter für die gemeinsame Nutzung eines {@link java.awt.event.ItemListener}.
     */
    private enum ConnectionParameter {
        /**
         * Portname.
         **/
        PORTNAME,
        /**
         * Baudrate.
         **/
        BAUDRATE,
        /**
         * Datenbits.
         **/
        DATABITS,
        /**
         * Stopbits.
         **/
        STOPBITS,
        /**
         * Parity.
         **/
        PARITY;
    }

    /**
     * Klasse ConnectionParameterComboBoxItemListener implementiert das Interface {@link java.awt.event.ItemListener}.
     * Diese Klasse stellt die funktionalität für das Event eines wechsel der Auswahl in der {@link JComboBox} bereit.
     */
    class ConnectionParameterComboBoxItemListener implements ItemListener {

        /**
         * Der Verbindungsparameter welcher verarbeitet werden soll.
         */
        private ConnectionParameter param;

        /**
         * Konstruktor.
         *
         * @param inParam der Verbindungsparameter {@link de.stefandraegersoftware.gui.mvc.Controller.ConnectionParameter} welcher verarbeitet werden soll.
         */
        ConnectionParameterComboBoxItemListener(final ConnectionParameter inParam) {
            this.param = inParam;
        }

        /**
         * @param e das {@link java.awt.event.ItemEvent}
         * @see java.awt.event.ItemListener#itemStateChanged(ItemEvent)
         */
        @Override
        public void itemStateChanged(final ItemEvent e) {
            switch (param) {
                case PORTNAME:
                    model.setPortname(e.getItem().toString());
                    break;
                case BAUDRATE:
                    model.setBaudrate(Integer.parseInt(e.getItem().toString()));
                    break;
                case DATABITS:
                    model.setDatabits(Integer.parseInt(e.getItem().toString()));
                    break;
                case STOPBITS:
                    model.setStopbits(Integer.parseInt(e.getItem().toString()));
                    break;
                case PARITY:
                    model.setParity(Integer.parseInt(e.getItem().toString()));
                    break;
                default:
                    throw new IllegalArgumentException("Unknown Value " + param.toString());
            }
        }
    }

    /**
     * Klasse TimerNumberFieldKeyListener implementiert das Interface {@link java.awt.event.KeyListener}.
     * Diese Klasse stellt sicher das nur Nummerische Eingaben (Zahlen von 0-9) bei der Eingabe zugelassen werden.
     */
    class TimerNumberFieldKeyListener implements KeyListener {

        @Override
        public void keyTyped(final KeyEvent e) {

        }

        @Override
        public void keyPressed(final KeyEvent e) {

        }

        @Override
        public void keyReleased(final KeyEvent e) {
            String textfieldValue = frame.getZeitTextfield().getText();
            for (int i = 0; i < textfieldValue.length(); i++) {
                char c = textfieldValue.charAt(i);
                if (!Character.isDigit(c)) {
                    Toolkit.getDefaultToolkit().beep();
                    frame.getZeitTextfield().setText(textfieldValue.substring(0, i));
                    model.setSleep(0);
                    break;
                } else {
                    model.setSleep(Long.parseLong(textfieldValue));
                }
            }
        }
    }
}
