package de.stefandraegersoftware.gui.mvc;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.MatteBorder;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.KeyListener;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.logging.Logger;

import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.BAUDRATE_LBL;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.BEENDEN_BTN;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.DATENBITS_LBL;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.ES_WURDE_KEIN_ANGESCHLOSSENES_GERAET_GEFUNDEN;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.FARBE_WAEHLEN_TITEL;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.HINWEIS;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.LEDFRAME_TITEL;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.PARITY_LBL;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.PORTNAME_LBL;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.STOPBITS_LBL;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.TEST_BTN;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.UEBERTRAGEN_BTN;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.VERBINDUNGSPARAMETER_TITEL;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.ZEIT_LBL;
import static de.stefandraegersoftware.gui.mvc.LEDFrameStatics.ZEIT_WAEHLEN_TITEL;

/**
 * View Klasse, zum Darstellen einer Oberfläche welche dem Benutzer erlaubt eine Verbindung mit dem Arduino aufzubauen,
 * es wird ein festdefinierter String gesendet welcher durch ein Programm auf dem Arduino dazu führt das eine 3 Farb SMD RGB LED aufleuchtet.
 * Generell ist jeder Farbton möglich jedoch sind je nach Modell die LED's zu weit auseinander so das ggf. mit einem LED Diffusor gearbeitet werden sollte.
 */
public class LEDFrame extends JFrame implements Observer {

    /**
     * Pfadname zum Bild 'led_small.png'.
     **/
    private static final String LED_SMALL_PNG = "images/led_small.png";
    /**
     * Pfadname zum Bild 'send.png'.
     **/
    private static final String SEND_PNG = "images/send.png";
    /**
     * Pfadname zum Bild 'exit.png'.
     **/
    private static final String EXIT_PNG = "images/exit.png";
    /**
     * Pfadname zum Bild 'test.png'.
     **/
    private static final String TEST_PNG = "images/test.png";
    /**
     * Pfadname zum Bild 'color_picker.png'.
     **/
    private static final String COLOR_PICKER_PNG = "images/color_picker.png";


    /**
     * {@link java.awt.Dimension} mit den Werten für die {@link javax.swing.JComboBox} der Verbindungsparamter.
     */
    private Dimension componentDimension = new Dimension(100, 18);

    /**
     * {@link javax.swing.JComboBox} für die Anzeige der verfügbaren Ports.
     */
    private JComboBox<String> availableComPortsComboBox;

    /**
     * {@link javax.swing.JButton} für das Uebertragen der Daten.
     */
    private JButton sendBtn;

    /**
     * {@link javax.swing.JButton} für das Beenden der Anwendung.
     */
    private JButton closeBtn;

    /**
     * {@link javax.swing.JButton} für das Wählen einer Farbe.
     */
    private JButton colorChooserBtn;

    /**
     * {@link javax.swing.JButton} für das Testen der Verbindungsparamerter.
     */
    private JButton testConnectionBtn;

    /**
     * {@link javax.swing.JComboBox} mit den zulässigen Werten für die Baudrate.
     */
    private JComboBox<Integer> availableBaudrateComboBox;

    /**
     * {@link javax.swing.JComboBox} mit den zulässigen Werten für die Datenbits.
     */
    private JComboBox<Integer> availableDatabitsComboBox;

    /**
     * {@link javax.swing.JComboBox} mit den zulässigen Werten für die Stopbits.
     */
    private JComboBox<Integer> availableStopbitsComboBox;

    /**
     * {@link javax.swing.JComboBox} mit den zulässigen Werten für die Parity.
     */
    private JComboBox<Integer> availableParityComboBox;

    /**
     * {@link javax.swing.JTextField} mit dem Wert für die Zeit.
     */
    private JTextField zeitTextfield;

    /**
     * {@link javax.swing.JPanel} zum visuellen darstellen der Farbe.
     */
    private JPanel colorChooserPanel;

    /**
     * {@link javax.swing.border.Border} des Zeit JTextField.
     */
    private  Border defaultTimerborder;

    /**
     * {@link java.util.logging.Logger} Objekt zum protokolieren von Fehler und Hinweisen aus dem Ablauf der Anwendung.
     */
    private static Logger logger = Logger.getLogger("LEDFrame");

    /**
     * Konstruktor.
     */
    public LEDFrame() {
        init();
        initComponents();
    }

    /**
     * Initialisiert das Frame.
     */
    private void init() {
        this.setTitle(LEDFRAME_TITEL);
        this.setIconImage(getApplicationImage());
        this.setBounds(0, 0, 500, 200);
        this.setLayout(new BorderLayout());
        this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        this.setLocationRelativeTo(null);
        this.setResizable(false);
    }

    /**
     * Erzeugt die Komponenten für das Frame.
     */
    private void initComponents() {
        JPanel westPnl = new JPanel();
        westPnl.setLayout(new FlowLayout());
        westPnl.add(getConnectionPanel());
        this.add(westPnl, BorderLayout.WEST);
        JPanel componentPnl = new JPanel();
        componentPnl.setLayout(new FlowLayout(FlowLayout.LEFT));
        componentPnl.add(getColorChooserPanel());
        componentPnl.add(getTimerPanel());
        this.add(componentPnl, BorderLayout.CENTER);
        JPanel btnPnl = new JPanel();
        btnPnl.setLayout(new FlowLayout(FlowLayout.RIGHT));
        btnPnl.setBorder(new MatteBorder(1, 0, 0, 0, Color.GRAY));
        btnPnl.add(getSendBtn());
        btnPnl.add(getCloseBtn());
        this.add(btnPnl, BorderLayout.SOUTH);
    }

    /**
     * Erzeugt die Schaltfläche Uebertragen.
     *
     * @return - {@link JButton}
     */
    private Component getSendBtn() {
        this.sendBtn = new JButton(UEBERTRAGEN_BTN);
        this.sendBtn.setIcon(new ImageIcon(getSendBtnImage()));
        return sendBtn;
    }

    /**
     * Erzeugt die Schaltfläche Beenden.
     *
     * @return - {@link JButton}
     */
    private Component getCloseBtn() {
        this.closeBtn = new JButton(BEENDEN_BTN);
        this.closeBtn.setIcon(new ImageIcon(getExitBtnImage()));
        return closeBtn;
    }

    /**
     * Erzeugt das Panel zum eintragen der Verbindungsparameter.
     *
     * @return - {@link JPanel}
     */
    private Component getConnectionPanel() {
        JPanel connectionPanel = new JPanel();
        connectionPanel.setBorder(new TitledBorder(VERBINDUNGSPARAMETER_TITEL));
        connectionPanel.setLayout(new GridLayout(0, 2));
        JLabel comPortLbl = new JLabel(PORTNAME_LBL);
        this.availableComPortsComboBox = new JComboBox<>();
        this.availableComPortsComboBox.setPreferredSize(componentDimension);

        JLabel baudrateLbl = new JLabel(BAUDRATE_LBL);
        this.availableBaudrateComboBox = new JComboBox<>();
        this.availableBaudrateComboBox.setPreferredSize(componentDimension);

        JLabel databitsLbl = new JLabel(DATENBITS_LBL);
        this.availableDatabitsComboBox = new JComboBox<>();
        this.availableDatabitsComboBox.setPreferredSize(componentDimension);

        JLabel stopbitsLbl = new JLabel(STOPBITS_LBL);
        this.availableStopbitsComboBox = new JComboBox<>();
        this.availableStopbitsComboBox.setPreferredSize(componentDimension);

        JLabel parityLbl = new JLabel(PARITY_LBL);
        this.availableParityComboBox = new JComboBox<>();
        this.availableParityComboBox.setPreferredSize(componentDimension);

        this.testConnectionBtn = new JButton(TEST_BTN);
        this.testConnectionBtn.setPreferredSize(componentDimension);
        this.testConnectionBtn.setIcon(new ImageIcon(getTestBtnImage()));
        connectionPanel.add(comPortLbl);
        connectionPanel.add(this.availableComPortsComboBox);
        connectionPanel.add(baudrateLbl);
        connectionPanel.add(this.availableBaudrateComboBox);
        connectionPanel.add(databitsLbl);
        connectionPanel.add(this.availableDatabitsComboBox);
        connectionPanel.add(stopbitsLbl);
        connectionPanel.add(this.availableStopbitsComboBox);
        connectionPanel.add(parityLbl);
        connectionPanel.add(this.availableParityComboBox);
        //connectionPanel.add(this.testConnectionBtn);
        return connectionPanel;
    }

    /**
     * Erzeugt das Panel zum wählen einer Farbe.
     *
     * @return - {@link JPanel}
     */
    private Component getColorChooserPanel() {
        JPanel colorPanel = new JPanel();
        colorPanel.setBorder(new TitledBorder(FARBE_WAEHLEN_TITEL));
        colorPanel.setLayout(new FlowLayout());
        this.colorChooserPanel = new JPanel();
        colorChooserPanel.setPreferredSize(new Dimension(75, 24));
        colorPanel.add(colorChooserPanel);

        this.colorChooserBtn = new JButton();
        this.colorChooserBtn.setIcon(new ImageIcon(getColorChooserBtnImage()));

        colorPanel.add(this.colorChooserBtn);
        return colorPanel;
    }

    /**
     * Erzeugt das Panel zum setzen der leuchtdauer in Millisekunden.
     *
     * @return - {@link JPanel}
     */
    private Component getTimerPanel() {
        JPanel timerPanel = new JPanel();
        timerPanel.setBorder(new TitledBorder(ZEIT_WAEHLEN_TITEL));
        timerPanel.setLayout(new FlowLayout());
        JLabel zeitLbl = new JLabel(ZEIT_LBL);
        this.zeitTextfield = new JTextField();
        this.defaultTimerborder = this.zeitTextfield.getBorder();
        this.zeitTextfield.setPreferredSize(new Dimension(80, 20));
        timerPanel.add(zeitLbl);
        timerPanel.add(this.zeitTextfield);
        return timerPanel;
    }

    protected void showNoSerialPortWasFoundMessage(){
        JOptionPane.showConfirmDialog(null, ES_WURDE_KEIN_ANGESCHLOSSENES_GERAET_GEFUNDEN, HINWEIS, JOptionPane.CLOSED_OPTION, JOptionPane.WARNING_MESSAGE);
    }

    /**
     * Setzt den {@link ActionListener} für die Schaltfläche "Farbe wählen".
     *
     * @param actionListener {@link ActionListener}
     */
    protected final void setColorChooserListener(final ActionListener actionListener) {
        this.colorChooserBtn.addActionListener(actionListener);
    }

    /**
     * Setzt den {@link ActionListener} für die Schaltfläche "Uebertragen".
     *
     * @param actionListener {@link ActionListener}
     */
    protected final void setSendListener(final ActionListener actionListener) {
        this.sendBtn.addActionListener(actionListener);
    }

    /**
     * Setzt den {@link ActionListener} für die Schaltfläche "Beenden".
     *
     * @param actionListener {@link ActionListener}
     */
    protected final void setCloseBtnListener(final ActionListener actionListener) {
        this.closeBtn.addActionListener(actionListener);
    }

    /**
     * Setzt den {@link ActionListener} für die Schaltfläche "Test".
     *
     * @param actionListener {@link ActionListener}
     */
    protected final void setTestBtnListener(final ActionListener actionListener) {
        this.testConnectionBtn.addActionListener(actionListener);
    }


    /**
     * Setzt den {@link ItemListener} für die Auswahlliste "Port".
     *
     * @param itemListener {@link ItemListener}
     */
    protected final void setPortComboBoxKeyListener(final ItemListener itemListener) {
        this.availableComPortsComboBox.addItemListener(itemListener);
    }

    /**
     * Setzt den {@link ItemListener} für die Auswahlliste "Baudrate".
     *
     * @param itemListener {@link ItemListener}
     */
    protected final void setBaudrateComboBoxListener(final ItemListener itemListener) {
        this.availableBaudrateComboBox.addItemListener(itemListener);
    }

    /**
     * Setzt den {@link ItemListener} für die Auswahlliste "Databits".
     *
     * @param itemListener {@link ItemListener}
     */
    protected final void setDatabitsComboBoxListener(final ItemListener itemListener) {
        this.availableDatabitsComboBox.addItemListener(itemListener);
    }

    /**
     * Setzt den {@link ItemListener} für die Auswahlliste "Stopbits".
     *
     * @param itemListener {@link ItemListener}
     */
    protected final void setStopbitsComboBoxListener(final ItemListener itemListener) {
        this.availableStopbitsComboBox.addItemListener(itemListener);
    }

    /**
     * Setzt den {@link ItemListener} für die Auswahlliste "Parity".
     *
     * @param itemListener {@link ItemListener}
     */
    protected final void setParityComboBoxListener(final ItemListener itemListener) {
        this.availableParityComboBox.addItemListener(itemListener);
    }

    /**
     * Setzt den {@link KeyListener} für das Eingabefeld "Zeit in Millisekunden:".
     *
     * @param keyListener {@link KeyListener}
     */
    protected final void setTimerTextFieldListener(final KeyListener keyListener) {
        this.zeitTextfield.addKeyListener(keyListener);
    }

    /**
     * Liefert die Auswahlliste der Ports.
     *
     * @return {@link JComboBox}
     */
    protected final JComboBox<String> getAvailableComPortsComboBox() {
        return this.availableComPortsComboBox;
    }

    /**
     * Liefert das TextFeld der Zeit.
     *
     * @return {@link JTextField}
     */
    protected final JTextField getZeitTextfield() {
        return this.zeitTextfield;
    }

    /**
     * Liefert die Auswahlliste der Baudrate.
     *
     * @return {@link JComboBox}
     */
    protected final JComboBox<Integer> getAvailableBaudrateComboBox() {
        return availableBaudrateComboBox;
    }

    /**
     * Liefert die Auswahlliste der Databits.
     *
     * @return {@link JComboBox}
     */
    protected final JComboBox<Integer> getAvailableDatabitsComboBox() {
        return availableDatabitsComboBox;
    }

    /**
     * Liefert die Auswahlliste der Stopbits.
     *
     * @return {@link JComboBox}
     */
    protected final JComboBox<Integer> getAvailableStopbitsComboBox() {
        return availableStopbitsComboBox;
    }

    /**
     * Liefert die Auswahlliste der Parity.
     *
     * @return {@link JComboBox}
     */
    protected final JComboBox<Integer> getAvailableParityComboBox() {
        return availableParityComboBox;
    }

    @Override
    public final void update(final Observable o, final Object arg) {
        if (arg instanceof Model) {
            Model model = (Model) arg;
            Map<String, Integer> colorMap = model.getColorMap();
            Color color = transformColorMapToColor(colorMap);
            this.colorChooserPanel.setBackground(transformColorMapToColor(colorMap));
            this.colorChooserPanel.setToolTipText(color.toString());
        }
    }

    /**
     * Transformiert die Map in ein gültiges {@link java.awt.Color} Objekt.
     *
     * @param map eine Map
     * @return liefert ein gültiges {@link java.awt.Color} Objekt
     */
    private Color transformColorMapToColor(final Map<String, Integer> map) {
        int red = map.get(Model.RED);
        int green = map.get(Model.GREEN);
        int blue = map.get(Model.BLUE);
        return new Color(red, green, blue);
    }

    protected void highlightTimerTextField(boolean highlight){
        if(highlight){
            this.zeitTextfield.setBorder(new MatteBorder(1,1,1,1,Color.RED));
        } else {
            this.zeitTextfield.setBorder(this.defaultTimerborder);
        }
    }

    /**
     * Liefert das Bild für das {@link JFrame} Objekt.
     *
     * @return - {@link java.awt.Image}
     */
    private Image getApplicationImage() {
        return getImage(LED_SMALL_PNG);
    }

    /**
     * Liefert das Bild für die Schaltfläche Uebertragen.
     *
     * @return - {@link java.awt.Image}
     */
    private Image getSendBtnImage() {
        return getImage(SEND_PNG);
    }

    /**
     * Liefert das Bild für die Schaltfläche Beenden.
     *
     * @return - {@link java.awt.Image}
     */
    private Image getExitBtnImage() {
        return getImage(EXIT_PNG);
    }

    /**
     * Liefert das Bild für die Schaltfläche Test.
     *
     * @return - {@link java.awt.Image}
     */
    private Image getTestBtnImage() {
        return getImage(TEST_PNG);
    }

    /**
     * Liefert das Bild für die Schaltfläche 'Farbe wählen'.
     *
     * @return - {@link java.awt.Image}
     */
    private Image getColorChooserBtnImage() {
        return getImage(COLOR_PICKER_PNG);
    }

    /**
     * Liefert ein {@link java.awt.Image} aus dem Maven Resources Ordner.
     *
     * @param path der Pfad zum Bild
     * @return gibt das {@link java.awt.Image} zurück.
     */
    private Image getImage(final String path) {
        return new ImageIcon(this.getClass().getClassLoader().getResource(path)).getImage();
    }
}
