package de.stefandraegersoftware.gui.mvc;


import de.stefandraegersoftware.SerialCommunication;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Klasse Model.
 * Die Klasse Model erweitert  die Klasse {@link java.util.Observable} und dient zum benachrichtigen einer Änderung an den Daten.
 */
public class Model extends Observable {

    /**
     * Membervariable für den Portnamen.
     */
    private String portname;

    /**
     * Membervariable für die Baudrate.
     */
    private int baudrate;

    /**
     * Membervariable für die Datenbits.
     */
    private int databits;

    /**
     * Membervariable für die Stopbits.
     */
    private int stopbits;

    /**
     * Membervariable für die Parity.
     */
    private int parity;

    /**
     * Membervariable für den Sleep Wert.
     */
    private long sleep;

    /**
     * Konstante für den Schlüssel der {@link java.util.Map} mit der Farbe - Rot.
     */
    protected static final String RED = "red";

    /**
     * Konstante für den Schlüssel der {@link java.util.Map} mit der Farbe - Grün.
     */
    protected static final String GREEN = "green";

    /**
     * Konstante für den Schlüssel der {@link java.util.Map} mit der Farbe - Blau.
     */
    protected static final String BLUE = "blue";

    /**
     * Die {@link java.util.Map} mit der Farbe.
     */
    private Map<String, Integer> colorMap;

    /**
     * {@link java.util.logging.Logger} Objekt.
     */
    private static Logger logger = Logger.getLogger("Model");

    /**
     * Konstruktor.
     */
    public Model() {
        colorMap = new HashMap<String, Integer>();
        colorMap.put(RED, 0);
        colorMap.put(GREEN, 0);
        colorMap.put(BLUE, 0);
    }

    /**
     * Liefert eine {@link java.util.List} mit den verfügbaren Portnamen.
     *
     * @return eine {@link java.util.List} mit den verfügbaren Portnamen.
     */
    public final List<String> getAvailablePortnames() {
        return SerialCommunication.getAvailablePortnameValues();
    }

    /**
     * Liefert eine {@link java.util.List} mit den zulässigen Werten für die Bausrate.
     *
     * @return eine {@link java.util.List} mit den zulässigen Werten für die Bausrate.
     */
    public final List<Integer> getAvailableBaudrate() {
        return SerialCommunication.getAvailableBaudrateValues();
    }

    /**
     * Liefert eine {@link java.util.List} mit den zulässigen Werten für die Datenbits.
     *
     * @return eine {@link java.util.List} mit den zulässigen Werten für die Datenbits.
     */
    public final List<Integer> getAvailableDatabits() {
        return SerialCommunication.getAvailableDatabitsValues();
    }

    /**
     * Liefert eine {@link java.util.List} mit den zulässigen Werten für die Stopbits.
     *
     * @return eine {@link java.util.List} mit den zulässigen Werten für die Stopbits.
     */
    public final List<Integer> getAvailableStopbits() {
        return SerialCommunication.getAvailableStopbitsValues();
    }

    /**
     * Liefert eine {@link java.util.List} mit den zulässigen Werten für die Parity.
     *
     * @return eine {@link java.util.List} mit den zulässigen Werten für die Parity.
     */
    public final List<Integer> getAvailableParity() {
        return SerialCommunication.getAvailableParityValues();
    }

    /**
     * Liefert den aktuellen Wert für die gewählte Baudrate.
     *
     * @return ein primitiven Integer Wert für die gewählte Baudrate.
     */
    public final int getBaudrate() {
        return baudrate;
    }

    /**
     * Setzt die Baudrate, und protokoliert dieses mit Logger auf der Konsole.
     * Nach dem setzen werden die angemeldeten Views über diese zustandsänderung an dem Model informiert. (Observer Pattern.)
     *
     * @param inBaudrate die gewählte Baudrate.
     */
    public final void setBaudrate(final int inBaudrate) {
        this.baudrate = inBaudrate;
        logger.log(Level.INFO, "set baudrate " + inBaudrate);
        notifyObservs();
    }

    /**
     * Setzt die Farbe, und protokoliert dieses mit Logger auf der Konsole.
     * Nach dem setzen werden die angemeldeten Views über diese zustandsänderung an dem Model informiert. (Observer Pattern.)
     *
     * @param inColorMap die gewählte Farbe als {@link java.util.Map}.
     */
    public final void setColorMap(final Map<String, Integer> inColorMap) {
        this.colorMap = inColorMap;
        logger.log(Level.INFO, "set colorMap " + inColorMap);
        notifyObservs();
    }

    /**
     * Liefert die gewählte Farbe als {@link java.util.Map}.
     *
     * @return die gewählte Farbe als {@link java.util.Map}.
     */
    public final Map<String, Integer> getColorMap() {
        return this.colorMap;
    }

    /**
     * Liefert die Datenbits.
     *
     * @return die Datenbits als primitiver Integer Wert.
     */
    public final int getDatabits() {
        return databits;
    }

    /**
     * Setzt den Wert für die Datenbits, und protokoliert dieses mit Logger auf der Konsole.
     * Nach dem setzen werden die angemeldeten Views über diese zustandsänderung an dem Model informiert. (Observer Pattern.)
     *
     * @param inDatabits der gewählte Wert für die Datenbits als primitiver Integer Wert.
     */
    public final void setDatabits(final int inDatabits) {
        this.databits = inDatabits;
        logger.log(Level.INFO, "set databits " + inDatabits);
        notifyObservs();
    }

    /**
     * Liefert den Wert für die Parity als primitiver Integer Wert.
     *
     * @return der Wert für die Parity als primitiver Integer Wert.
     */
    public final int getParity() {
        return parity;
    }

    /**
     * Setzt den Wert für die Parity, und protokoliert dieses mit Logger auf der Konsole.
     * Nach dem setzen werden die angemeldeten Views über diese zustandsänderung an dem Model informiert. (Observer Pattern.)
     *
     * @param inParity der gewählte Wert für die Parity als primitiver Integer Wert.
     */
    public final void setParity(final int inParity) {
        this.parity = inParity;
        logger.log(Level.INFO, "set parity " + inParity);
        notifyObservs();
    }

    /**
     * Liefert den Portnamen.
     *
     * @return der gewählte Portname als {@link java.lang.String} Wert.
     */
    public final String getPortname() {
        return portname;
    }

    /**
     * Setzt den Portnamen.
     *
     * @param inPortname der Portname
     */
    public final void setPortname(final String inPortname) {
        this.portname = inPortname;
        logger.log(Level.INFO, "set portname " + inPortname);
        notifyObservs();
    }

    /**
     * Liefert den Wert für die aufleuchtdauer der LED.
     *
     * @return der Wert für die aufleuchtdauer der LED als primitiver {@link java.lang.Long} Wert.
     */
    public final long getSleep() {
        return sleep;
    }

    /**
     * Setzt den Wert für die aufleuchtdauer der LED, und protokoliert dieses mit Logger auf der Konsole.
     * Nach dem setzen werden die angemeldeten Views über diese zustandsänderung an dem Model informiert. (Observer Pattern.)
     *
     * @param inSleep der gewählte Wert für die aufleuchtdauer der LED als primitiver Long Wert.
     */
    public final void setSleep(final long inSleep) {
        this.sleep = inSleep;
        logger.log(Level.INFO, "set sleep " + inSleep);
        notifyObservs();
    }

    /**
     * Liefert den gewählten Wert für die Stopbits.
     *
     * @return der gewählte Wert für die Stopbits als primitiver Integer Wert.
     */
    public final int getStopbits() {
        return stopbits;
    }

    /**
     * Setzt den Wert für die Stopbits, und protokoliert dieses mit Logger auf der Konsole.
     * Nach dem setzen werden die angemeldeten Views über diese zustandsänderung an dem Model informiert. (Observer Pattern.)
     *
     * @param inStopbits der gewählte Wert für die Stopbits als primitiver Integer Wert.
     */
    public final void setStopbits(final int inStopbits) {
        this.stopbits = inStopbits;
        logger.log(Level.INFO, "set stopbits " + inStopbits);
        notifyObservs();
    }

    /**
     * Methode zum benachrichten der angemeldeten Observers.
     */
    private void notifyObservs() {
        setChanged();
        notifyObservers(this);
    }
}
